package com.jsh.erp.datasource.mappers;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.github.pagehelper.PageHelper;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.result.PageParam;
import com.jsh.erp.result.PageResult;
import com.jsh.erp.util.MyBatisUtils;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;

/**
 * 在 MyBatis Plus 的 BaseMapper 的基础上拓展，提供更多的能力
 */
public interface BaseMapperX<T> extends BaseMapper<T> {

    default PageResult<T> selectPage(PageParam pageParam, @Param("ew") Wrapper<T> queryWrapper) {
        // MyBatis Plus 查询
        IPage<T> mpPage = MyBatisUtils.buildPage(pageParam);
        selectPage(mpPage, queryWrapper);
        // 转换返回
        return new PageResult<T>(mpPage.getRecords(), mpPage.getTotal(),(int)mpPage.getSize(),(int)mpPage.getCurrent());
    }

    default T selectOne(String field, Object value) {
        return selectOne(new QueryWrapper<T>().eq(field, value));
    }

    /**
     *  获取查询mapper
     * @return
     */
    default LambdaQueryWrapperX<T> getWeekend() {
        return new LambdaQueryWrapperX<>();
    }

    /**
     *  自定义分页
     * @param supplier
     * @param pageQuery
     * @param <T>
     * @return
     */
    default <T> PageResult<T> pageSelect(Supplier<List<T>> supplier, PageParam pageQuery) {
        if (Objects.equals(pageQuery.getPageNo(), 0)) {
            List<T> list = supplier.get();
            return PageResult.of(list, pageQuery);
        } else {
            com.github.pagehelper.Page<T> page = PageHelper.startPage(pageQuery.getPageNo(), pageQuery.getPageSize());
            List<T> list = supplier.get();
            PageResult<T> resultVO = PageResult.of(list, pageQuery);
            resultVO.setTotal(page.getTotal());
            resultVO.setPageNumber(page.getPageNum());
            resultVO.setPageSize(page.getPageSize());
            return resultVO;
        }
    }

    default T selectOne(SFunction<T, ?> field, Object value) {
        return selectOne(new LambdaQueryWrapper<T>().eq(field, value));
    }

    default T selectOne(String field1, Object value1, String field2, Object value2) {
        return selectOne(new QueryWrapper<T>().eq(field1, value1).eq(field2, value2));
    }

    default T selectOne(SFunction<T, ?> field1, Object value1, SFunction<T, ?> field2, Object value2) {
        return selectOne(new LambdaQueryWrapper<T>().eq(field1, value1).eq(field2, value2));
    }

    default Long selectCount() {
        return selectCount(new QueryWrapper<T>());
    }

    default Long selectCount(String field, Object value) {
        return selectCount(new QueryWrapper<T>().eq(field, value));
    }

    default Long selectCount(SFunction<T, ?> field, Object value) {
        return selectCount(new LambdaQueryWrapper<T>().eq(field, value));
    }

    default Long selectCount(SFunction<T, ?> field1, Object value1, SFunction<T, ?> field2, Object value2) {
        return selectCount(new LambdaQueryWrapper<T>().eq(field1, value1).eq(field2, value2));
    }

    default List<T> selectList() {
        return selectList(new QueryWrapper<>());
    }

    default List<T> selectList(String field, Object value) {
        return selectList(new QueryWrapper<T>().eq(field, value));
    }

    default List<T> selectList(SFunction<T, ?> field, Object value, SFunction<T, ?> field1, Object value1) {
        return selectList(new LambdaQueryWrapper<T>().eq(field, value).eq(field1, value1));
    }

    default List<T> selectList(SFunction<T, ?> field, Object value) {
        return selectList(new LambdaQueryWrapper<T>().eq(field, value));
    }

    default List<T> selectList(String field, Collection<?> values) {
        return selectList(new QueryWrapper<T>().in(field, values));
    }

    default List<T> selectList(SFunction<T, ?> field, Collection<?> values) {
        return selectList(new LambdaQueryWrapper<T>().in(field, values));
    }
    
    default int deleteByProperty(SFunction<T, ?> field, Object value) {
        if(value == null){
            return -1;
        }
        return delete(new LambdaQueryWrapperX<T>().eq(field,value));
    }

    /**
     * 逐条插入，适合少量数据插入，或者对性能要求不高的场景
     *
     * 如果大量，请使用 {@link com.baomidou.mybatisplus.extension.service.impl.ServiceImpl#saveBatch(Collection)} 方法
     * 使用示例，可见 RoleMenuBatchInsertMapper、UserRoleBatchInsertMapper 类
     *
     * @param entities 实体们
     */
    default void insertBatch(Collection<T> entities) {
        entities.forEach(this::insert);
    }

    default void updateBatch(T update) {
        update(update, new QueryWrapper<>());
    }

}
